@@ -1,5 +1,3 @@
-# encoding: utf-8
-
 # This file is part of CycloneDX Python Lib
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -17,14 +15,27 @@
 # SPDX-License-Identifier: Apache-2.0
 # Copyright (c) OWASP Foundation. All Rights Reserved.
 
+
+from typing import Callable
 from unittest import TestCase
+from uuid import uuid4
 
-from cyclonedx.model import License, LicenseChoice, OrganizationalContact, OrganizationalEntity, Property
-from cyclonedx.model.bom import Bom, BomMetaData, ThisTool, Tool
+from ddt import ddt, named_data
+
+from cyclonedx.exception.model import LicenseExpressionAlongWithOthersException
+from cyclonedx.model import OrganizationalContact, OrganizationalEntity, Property, ThisTool, Tool
+from cyclonedx.model.bom import Bom, BomMetaData
+from cyclonedx.model.bom_ref import BomRef
 from cyclonedx.model.component import Component, ComponentType
-
-from .data import (
+from cyclonedx.model.license import DisjunctiveLicense
+from tests._data.models import (
+    get_bom_component_licenses_invalid,
+    get_bom_component_nested_licenses_invalid,
     get_bom_for_issue_275_components,
+    get_bom_metadata_component_licenses_invalid,
+    get_bom_metadata_component_nested_licenses_invalid,
+    get_bom_metadata_licenses_invalid,
+    get_bom_service_licenses_invalid,
     get_bom_with_component_setuptools_with_vulnerability,
     get_component_setuptools_simple,
     get_component_setuptools_simple_no_version,
@@ -58,12 +69,12 @@ class TestBomMetaData(TestCase):
         manufacturer = OrganizationalEntity(name='test_manufacturer')
         supplier = OrganizationalEntity(name='test_supplier')
         licenses = [
-            LicenseChoice(license_=License(spdx_license_id='MIT')),
-            LicenseChoice(license_=License(spdx_license_id='Apache-2.0')),
+            DisjunctiveLicense(id='MIT'),
+            DisjunctiveLicense(id='Apache-2.0'),
         ]
         properties = [
             Property(name='property_1', value='value_1'),
-            Property(name='property_2', value='value_2',)
+            Property(name='property_2', value='value_2', )
         ]
 
         metadata = BomMetaData(tools=tools, authors=authors, component=component,
@@ -87,6 +98,7 @@ class TestBomMetaData(TestCase):
         self.assertTrue(tools[1] in metadata.tools)
 
 
+@ddt
 class TestBom(TestCase):
 
     def test_bom_metadata_tool_this_tool(self) -> None:
@@ -100,28 +112,56 @@ class TestBom(TestCase):
         bom.metadata.tools.add(
             Tool(vendor='TestVendor', name='TestTool', version='0.0.0')
         )
+        self.assertEqual(bom.version, 1)
         self.assertEqual(len(bom.metadata.tools), 2)
 
     def test_metadata_component(self) -> None:
         metadata = Bom().metadata
         self.assertTrue(metadata.component is None)
-        hextech = Component(name='Hextech', version='1.0.0', component_type=ComponentType.LIBRARY)
+        hextech = Component(name='Hextech', version='1.0.0', type=ComponentType.LIBRARY)
         metadata.component = hextech
         self.assertFalse(metadata.component is None)
         self.assertEqual(metadata.component, hextech)
 
     def test_empty_bom(self) -> None:
         bom = Bom()
-        self.assertIsNotNone(bom.uuid)
+        self.assertEqual(bom.version, 1)
+        self.assertIsNotNone(bom.serial_number)
         self.assertIsNotNone(bom.metadata)
         self.assertFalse(bom.components)
         self.assertFalse(bom.services)
         self.assertFalse(bom.external_references)
 
+    def test_empty_bom_defined_serial(self) -> None:
+        serial_number = uuid4()
+        bom = Bom(serial_number=serial_number)
+        self.assertEqual(bom.serial_number, serial_number)
+        self.assertEqual(bom.get_urn_uuid(), serial_number.urn)
+        self.assertEqual(bom.version, 1)
+        self.assertEqual(bom.urn(), f'urn:cdx:{serial_number}/1')
+
+    def test_empty_bom_defined_serial_and_version(self) -> None:
+        serial_number = uuid4()
+        bom = Bom(serial_number=serial_number, version=2)
+        self.assertEqual(bom.serial_number, serial_number)
+        self.assertEqual(bom.get_urn_uuid(), serial_number.urn)
+        self.assertEqual(bom.version, 2)
+        self.assertEqual(bom.urn(), f'urn:cdx:{serial_number}/2')
+
     def test_bom_with_vulnerabilities(self) -> None:
         bom = get_bom_with_component_setuptools_with_vulnerability()
         self.assertTrue(bom.has_vulnerabilities())
 
+    def test_bom_get_vulnerabilities_by_bom_ref(self) -> None:
+        bom = get_bom_with_component_setuptools_with_vulnerability()
+        vulns = bom.get_vulnerabilities_for_bom_ref(bom_ref=BomRef(value='pkg:pypi/setuptools@50.3.2?extension=tar.gz'))
+        self.assertEqual(len(vulns), 1)
+
+    def test_bom_get_vulnerabilities_by_bom_ref_negative(self) -> None:
+        bom = get_bom_with_component_setuptools_with_vulnerability()
+        vulns = bom.get_vulnerabilities_for_bom_ref(bom_ref=BomRef(value='pkg:pypi/setuptools@50.3.1?extension=tar.gz'))
+        self.assertEqual(len(vulns), 0)
+
     def test_bom_nested_components_issue_275(self) -> None:
         """regression test for issue #275
         see https://github.com/CycloneDX/cyclonedx-python-lib/issues/275
@@ -131,6 +171,19 @@ class TestBom(TestCase):
         self.assertEqual(2, len(bom.components))
         bom.validate()
 
+    @named_data(
+        ['metadata_licenses', get_bom_metadata_licenses_invalid],
+        ['metadata_component_licenses', get_bom_metadata_component_licenses_invalid],
+        ['metadata_component_nested_licenses', get_bom_metadata_component_nested_licenses_invalid],
+        ['component_licenses', get_bom_component_licenses_invalid],
+        ['component_nested_licenses', get_bom_component_nested_licenses_invalid],
+        ['service_licenses', get_bom_service_licenses_invalid],
+    )
+    def test_validate_with_invalid_license_constellation_throws(self, get_bom: Callable[[], Bom]) -> None:
+        bom = get_bom()
+        with self.assertRaises(LicenseExpressionAlongWithOthersException):
+            bom.validate()
+
     # def test_bom_nested_services_issue_275(self) -> None:
     #    """regression test for issue #275
     #    see https://github.com/CycloneDX/cyclonedx-python-lib/issues/275
