@@ -19,6 +19,7 @@ import re
 from typing import Any, Callable
 from unittest import TestCase
 from unittest.mock import Mock, patch
+from warnings import warn
 
 from ddt import ddt, idata, named_data, unpack
 
@@ -28,7 +29,7 @@ from cyclonedx.model.bom import Bom
 from cyclonedx.output.xml import BY_SCHEMA_VERSION, Xml
 from cyclonedx.schema import OutputFormat, SchemaVersion
 from cyclonedx.validation.xml import XmlValidator
-from tests import SnapshotMixin, mksname, uuid_generator
+from tests import SnapshotMixin, mksname
 from tests._data.models import all_get_bom_funct_invalid, all_get_bom_funct_valid, bom_all_same_bomref
 
 
@@ -40,7 +41,6 @@ class TestOutputXml(TestCase, SnapshotMixin):
     ))
     @unpack
     @patch('cyclonedx.model.ThisTool._version', 'TESTING')
-    @patch('cyclonedx.model.bom_ref.uuid4', side_effect=uuid_generator(0, version=4))
     def test_valid(self, get_bom: Callable[[], Bom], sv: SchemaVersion, *_: Any, **__: Any) -> None:
         snapshot_name = mksname(get_bom, sv, OutputFormat.XML)
         bom = get_bom()
@@ -48,8 +48,10 @@ class TestOutputXml(TestCase, SnapshotMixin):
         try:
             errors = XmlValidator(sv).validate_str(xml)
         except MissingOptionalDependencyException:
-            errors = None  # skipped validation
-        self.assertIsNone(errors)
+            warn('!!! skipped schema validation',
+                 category=UserWarning, stacklevel=0)
+        else:
+            self.assertIsNone(errors)
         self.assertEqualSnapshot(xml, snapshot_name)
 
     @named_data(*(
